import { Logger } from '@nodescript/logger';
import { IncomingMessage, Server, ServerResponse } from 'http';
import { Socket } from 'net';
import { HttpContext } from './HttpContext.js';
import { HttpNext } from './HttpHandler.js';
export interface HttpServerConfig {
    port: number;
    address: string;
    socketTimeout: number;
    shutdownDelay: number;
    requestBodyLimitBytes: number;
}
export declare abstract class HttpServer {
    HTTP_PORT: number;
    HTTP_ADDRESS: string;
    HTTP_TIMEOUT: number;
    HTTP_SHUTDOWN_DELAY: number;
    HTTP_REQUEST_BODY_LIMIT_BYTES: number;
    logger: Logger;
    protected config: HttpServerConfig;
    protected server: Server | null;
    protected requestsPerSocket: Map<Socket, number>;
    protected stopping: boolean;
    constructor();
    abstract handle(ctx: HttpContext, next: HttpNext): Promise<void>;
    start(): Promise<void>;
    stop(): Promise<void>;
    getServer(): Server<typeof IncomingMessage, typeof ServerResponse> | null;
    protected handleRequest(req: IncomingMessage, res: ServerResponse): Promise<void>;
    /**
     * Tracks the number of pending requests per socket, so that the server could close gracefully
     */
    protected onConnection(sock: Socket): void;
    /**
     * Tracks requests per connection, close connection during shutdown, when requests are served.
     */
    protected onRequest(req: IncomingMessage, res: ServerResponse): void;
    protected getSocketRequests(socket: Socket): number;
    /**
     * Gracefully close idle sockets.
     */
    protected closeIdleSockets(): void;
    /**
     * Destroy all sockets non-gracefully.
     */
    protected destroyAllSockets(): void;
}
