var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { config } from 'mesh-config';
/**
 * Sets Access-Control-* headers and responds to CORS preflight requests.
 *
 * Note: it should be bound to global scope because doesn't use request-scoped dependencies.
 */
export class HttpCorsHandler {
    async handle(ctx, next) {
        ctx.setResponseHeader('Vary', 'Origin');
        const origin = ctx.state.corsOrigin ?? ctx.getRequestHeader('Origin');
        if (!origin) {
            return next();
        }
        const maxAge = ctx.state.corsMaxAge ?? this.CORS_MAX_AGE;
        const exposeHeaders = ctx.state.corsExposeHeaders ?? this.CORS_EXPOSE_HEADERS;
        const allowCredentials = ctx.state.corsAllowCredentials ?? this.CORS_ALLOW_CREDENTIALS;
        ctx.setResponseHeader('Access-Control-Allow-Origin', origin);
        ctx.setResponseHeader('Access-Control-Max-Age', String(maxAge));
        ctx.setResponseHeader('Access-Control-Expose-Headers', exposeHeaders);
        ctx.setResponseHeader('Access-Control-Allow-Credentials', String(allowCredentials));
        // Respond with 204 to preflight request
        const isPreflight = ctx.method === 'OPTIONS' && !!ctx.getRequestHeader('Access-Control-Request-Method');
        if (isPreflight) {
            const allowedMethods = ctx.state.corsAllowedMethods ?? ['GET', 'HEAD', 'POST', 'PUT', 'DELETE'];
            const allowedHeaders = ctx.getRequestHeader('Access-Control-Request-Headers');
            ctx.setResponseHeader('Access-Control-Allow-Methods', allowedMethods.join(','));
            ctx.setResponseHeader('Access-Control-Allow-Headers', allowedHeaders);
            ctx.status = 204;
            return;
        }
        return next();
    }
}
__decorate([
    config({ default: 'Content-Length,Date' }),
    __metadata("design:type", String)
], HttpCorsHandler.prototype, "CORS_EXPOSE_HEADERS", void 0);
__decorate([
    config({ default: 3600 }),
    __metadata("design:type", Number)
], HttpCorsHandler.prototype, "CORS_MAX_AGE", void 0);
__decorate([
    config({ default: true }),
    __metadata("design:type", Boolean)
], HttpCorsHandler.prototype, "CORS_ALLOW_CREDENTIALS", void 0);
//# sourceMappingURL=HttpCorsHandler.js.map