import { Logger } from '@nodescript/logger';
import { DomainMethodStat, ProtocolIndex } from '@nodescript/protocomm';
import { Event } from 'nanoevent';
import { HttpContext } from '../HttpContext.js';
import { HttpHandler, HttpNext } from '../HttpHandler.js';
/**
 * Routes HTTP requests according to specified protocol.
 *
 * Handler must be created in request scope by passing protocol implementation
 * (which can also request-scoped).
 */
export declare abstract class HttpProtocolHandler<P> implements HttpHandler {
    logger: Logger;
    abstract protocol: ProtocolIndex<P>;
    abstract protocolImpl: P;
    prefix: string;
    methodStats: Event<DomainMethodStat>;
    handle(ctx: HttpContext, next: HttpNext): Promise<void>;
    protected parsePath(path: string): string[];
    protected parseParams(ctx: HttpContext): Promise<any>;
}
