var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { NotFoundError } from '@nodescript/errors';
import { Logger } from '@nodescript/logger';
import { dep } from 'mesh-ioc';
import { Event } from 'nanoevent';
/**
 * Routes HTTP requests according to specified protocol.
 *
 * Handler must be created in request scope by passing protocol implementation
 * (which can also request-scoped).
 */
export class HttpProtocolHandler {
    constructor() {
        this.prefix = '';
        this.methodStats = new Event();
    }
    async handle(ctx, next) {
        const [domainName, methodName] = this.parsePath(ctx.url.pathname);
        const { methodDef, reqSchema, resSchema, } = this.protocol.lookupMethod(domainName, methodName);
        const startedAt = Date.now();
        try {
            const method = methodDef.type === 'query' ? 'GET' : 'POST';
            if (ctx.method !== method) {
                return await next();
            }
            const params = await this.parseParams(ctx);
            const decodedParams = reqSchema.decode(params, { strictRequired: true });
            this.logger.debug(`>>> ${domainName}.${methodName}`, decodedParams);
            const domainImpl = this.protocolImpl[domainName];
            const methodImpl = domainImpl[methodName];
            ctx.state.domainName = domainName;
            ctx.state.methodName = methodName;
            ctx.state.methodDef = methodDef;
            ctx.state.params = decodedParams;
            const res = await methodImpl.call(domainImpl, decodedParams);
            const decodedRes = resSchema.decode(res);
            this.logger.debug(`<<< ${domainName}.${methodName}`, decodedRes);
            ctx.status = 200;
            ctx.responseBody = decodedRes;
            this.methodStats.emit({
                domain: domainName,
                method: methodName,
                latency: Date.now() - startedAt,
            });
        }
        catch (error) {
            this.methodStats.emit({
                domain: domainName,
                method: methodName,
                latency: Date.now() - startedAt,
                error: error.name,
            });
            throw error;
        }
    }
    parsePath(path) {
        if (!path.startsWith(this.prefix)) {
            throw new NotFoundError('Endpoint not found');
        }
        path = path.substring(this.prefix.length);
        const m = /^\/([a-zA-Z0-9]+)\/([a-zA-Z0-9]+)$/.exec(path);
        if (!m) {
            throw new NotFoundError('Endpoint not found');
        }
        return [m[1], m[2]];
    }
    async parseParams(ctx) {
        if (ctx.method === 'GET') {
            return ctx.query;
        }
        const body = await ctx.readRequestBody();
        return body ?? {};
    }
}
__decorate([
    dep(),
    __metadata("design:type", Logger)
], HttpProtocolHandler.prototype, "logger", void 0);
//# sourceMappingURL=HttpProtocolHandler.js.map