import { matchPath } from '@nodescript/pathmatcher';
export const TypedArray = Object.getPrototypeOf(Uint8Array);
export function headersToDict(headers) {
    const dict = Object.create(null);
    for (const [key, value] of Object.entries(headers)) {
        const val = Array.isArray(value) ? value : [value ?? ''];
        dict[key.toLowerCase()] = val;
    }
    return dict;
}
export function searchParamsToDict(search) {
    const dict = Object.create(null);
    for (const [key, value] of search) {
        const values = dict[key] ?? [];
        values.push(value);
        dict[key] = values;
    }
    return dict;
}
export function composeHandlers(handlers) {
    const fns = handlers.map(_ => {
        return (ctx, next) => _.handle(ctx, next);
    });
    return {
        handle: compose(fns),
    };
}
export function compose(fns) {
    return async (ctx, next) => {
        const dispatch = async (index) => {
            if (index >= fns.length) {
                return await next();
            }
            const fn = fns[index];
            await fn(ctx, () => dispatch(index + 1));
        };
        return await dispatch(0);
    };
}
export function createRouteHandler(route) {
    return async (ctx, next) => {
        const [routeMethod, routePath, routeHandler] = route;
        const methodMatch = routeMethod === '*' || routeMethod === ctx.method;
        if (!methodMatch) {
            return next();
        }
        const pathParams = matchPath(routePath, ctx.url.pathname);
        if (!pathParams) {
            return next();
        }
        Object.assign(ctx.params, pathParams);
        await routeHandler(ctx, next);
    };
}
//# sourceMappingURL=util.js.map